# -*- coding: utf-8 -*-
from django.contrib.postgres.fields import JSONField
from django.db import models
from django.template.defaultfilters import truncatechars


class CSPReport(models.Model):
    host = models.TextField(blank=True, null=True, default='')
    document_uri = models.TextField()
    blocked_uri = models.TextField()
    referrer = models.TextField(blank=True, null=True, default='')

    body = JSONField(default={}, blank=True, null=True)
    request_meta = JSONField(default={}, blank=True, null=True)

    date = models.DateTimeField(auto_now_add=True)

    @property
    def short_blocked_uri(self):
        return truncatechars(self.blocked_uri, 100)

    def __str__(self):
        return 'CSPReport {}'.format(self.id)


class CleanedCSPReport(models.Model):
    report = models.ForeignKey(CSPReport)

    document_scheme = models.TextField(blank=True, null=True, default='')
    document_domain = models.TextField(blank=True, null=True, default='')
    document_path = models.TextField(blank=True, null=True, default='')
    document_query = models.TextField(blank=True, null=True, default='')

    blocked_domain = models.TextField(blank=True, null=True, default='')
    blocked_url = models.TextField(blank=True, null=True, default='')

    @property
    def short_blocked_uri(self):
        return truncatechars(self.blocked_url, 100)


class CSPReportFilter(models.Model):
    filter_type = models.TextField()
    value = models.TextField()
    is_enabled = models.BooleanField(default=True)

    def __str__(self):
        return '{}: {}'.format(self.filter_type, self.value)



from urllib.parse import urlparse
from csp_report.models import CSPReport, CleanedCSPReport, CSPReportFilter
from django.conf import settings

BLOCKED_URL_FILTERS = [
    {'type': 'equals', 'value': 'data'},
    {'type': 'equals', 'value': 'asset'},
]


def write_csp_report(report_data):
    return CSPReport.objects.create(**report_data)


def write_cleaned_csp_report(report):
    csp_blocked_url_filters = getattr(settings, 'CSP_BLOCKED_URL_FILTERS', BLOCKED_URL_FILTERS)

    cspreportfilters = CSPReportFilter.objects.filter(is_enabled=True).values()
    from_db_filter = [{'type': _['filter_type'], 'value': _['value']} for _ in cspreportfilters]

    for _ in csp_blocked_url_filters + from_db_filter:
        filter_type, filter_value = _['type'], _['value']
        if filter_type == 'equals':
            if report.blocked_uri == filter_value:
                return
        elif filter_type == 'startswith':
            if report.blocked_uri.startswith(filter_value):
                return

    parse_document_uri = urlparse(report.document_uri)
    parse_blocked_uri = urlparse(report.blocked_uri)

    report_data = {
        'report': report,
        'document_scheme': parse_document_uri.scheme,
        'document_domain': parse_document_uri.netloc,
        'document_path': parse_document_uri.path,
        'document_query': parse_document_uri.query,
        'blocked_domain': parse_blocked_uri.netloc,
        'blocked_url': parse_blocked_uri.path + parse_blocked_uri.query,
    }

    return CleanedCSPReport.objects.create(**report_data)


def get_important_request_meta(headers, important_headers=None):
    if not important_headers:
        important_headers = [
            'HTTP_USER_AGENT',
            'HTTP_X_FORWARDED_FOR',
            'HTTP_X_REAL_IP',
            'REMOTE_ADDR',
            'HTTP_ORIGIN',
            'HTTP_CACHE_CONTROL',
            'SERVER_NAME',
        ]
    return {k: v for k, v in headers.items() if k in important_headers}


def process_scp_report(data, **kwargs):
    request_meta = kwargs.get('request_meta', {})
    report_data = dict(
        host=kwargs.get('host', ''),
        document_uri=data['document-uri'],
        blocked_uri=data['blocked-uri'],
        referrer=data.get('referrer', ''),
        body=data,
        request_meta=get_important_request_meta(request_meta),
    )

    report = write_csp_report(report_data)

    write_cleaned_csp_report(report)


# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-25 13:22
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CSPReport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('host', models.TextField(blank=True, default='', null=True)),
                ('document_uri', models.TextField()),
                ('blocked_uri', models.TextField()),
                ('referrer', models.TextField(blank=True, default='', null=True)),
                ('body', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default={}, null=True)),
                ('request_meta', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default={}, null=True)),
                ('date', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



from datetime import timedelta, datetime

from django.core.management import BaseCommand
from csp_report.models import CSPReport


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            '--max_age',
            default=7,
            type=int,
            help='Number of days to store data',
        )

    def handle(self, *args, **options):
        max_age = options.get('max_age')
        if max_age:
            reports = CSPReport.objects.filter(date__lte=datetime.now()-timedelta(days=max_age))
            print('Trying to remove {} reports'.format(len(reports)))
            reports.delete()



# -*- coding: utf-8 -*-
import json
import logging

from csp_report.utils import process_scp_report
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_http_methods


logger = logging.getLogger('django.request')


@require_http_methods(['POST'])
@csrf_exempt
def csp_report_view(request):
    try:
        data = json.loads(request.body.decode())['csp-report']
        logger.debug(data)
        logger.debug(request.META)
        host = request.META.get('HTTP_HOST')
        process_scp_report(data, host=host, request_meta=dict(request.META))
    except KeyError:
        return HttpResponse(status=422)

    return HttpResponse(content='ok')



from distutils.core import setup

from setuptools import find_packages

setup(
  name='django-cspreport',
  packages=find_packages(),
  version='0.1.9',
  description='Django app for handling reports from web browsers of violations of content security policy.',
  author='Anton Tuchak',
  author_email='anton.tuchak@gmail.com',
  url='https://github.com/atuchak/django-cspreport',
  # download_url='https://github.com/atuchak/django-cspreport/archive/0.1.0.tar.gz',
  keywords=['Content-Security-Policy', 'csp', 'Django'],
  classifiers=[],
)


from django.contrib import admin

from csp_report.models import CSPReport, CleanedCSPReport, CSPReportFilter
from django.core import urlresolvers


class CSPReportAdmin(admin.ModelAdmin):
    list_display = ('id', 'host', 'document_uri', 'date', 'short_blocked_uri', )
    ordering = ['-id', ]
    list_filter = ('host', 'date', )

admin.site.register(CSPReport, CSPReportAdmin)


class CleanedCSPReportAdmin(admin.ModelAdmin):
    list_display = (
        'id',
        'link_to_report',
        'document_domain',
        'document_path',
        'document_query',
        'blocked_domain',
        'short_blocked_url',
    )
    ordering = ['-id', ]
    list_filter = ('document_domain', 'blocked_domain', 'document_path')

    def link_to_report(self, obj):
        link = urlresolvers.reverse('admin:csp_report_cspreport_change', args=[obj.report.id])
        return '<a href="{}">{}</a>'.format(link, obj.report.id)

    link_to_report.allow_tags = True

admin.site.register(CleanedCSPReport, CleanedCSPReportAdmin)


class CSPReportFilterAdmin(admin.ModelAdmin):
    ordering = ['-id', ]

admin.site.register(CSPReportFilter, CSPReportFilterAdmin)